"""Base constants like BUTTON_TYPES AND PROGRESS_TYPES"""
_WIDGET_TYPES = [
    "default",
    "primary",
    "success",
    "warning",
    "danger",
]
BUTTON_TYPES = _WIDGET_TYPES
PROGRESS_TYPES = _WIDGET_TYPES
