"""The Fast Templates provide easy to use, awesome looking templates for Panel based on the
https://fast.design framework"""
# pylint: disable=line-too-long
from awesome_panel_extensions.frameworks.fast.templates.fast_grid_template.fast_grid_template import (
    FastGridTemplate,
)
from awesome_panel_extensions.frameworks.fast.templates.fast_list_template.fast_list_template import (
    FastListTemplate,
)
